/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
 
/* $Header: Z:\cdasas\engine\core\h\iod.h   \main\8   Fri Aug  6 16:25:13 2004   de11417 $ */
/**********************************************************************
Filename:iod.h

File Description
	This file has data structures OSD and IOD and related #defines.

	OSD is used for converting a OS specific request to OS independent
	request packet. An OS specific request may break into more than one
	OSD only if OS sends more scatter gather elements than the SGList of
	each OSD can handle. OSDs are also internally generated for rescan or
	internal IOCTL calls or from IOMGR.

	IOD is used to convert one OS independent request into one or more
	drive specific requests. e.g. A write on RAID1 will create two IODs
	for each OSD. IODs are also used to issue internal IO commands such
	as verify, rebuild, recovery, meta data read/writes etc.

	Each IOD has an embedded IOB what CHIM below us understands and uses.
	
	We also use the IOD fields to create different types of queues that this
	IOD belongs to. e.g. Meta Data functions use a queue to implement re-entrancy
	and for that a field is defined in IOD to chain them.

	A call back stack for post processing is maintained within each IOD.
	
	Notes about Post processing stack of routines and iIOD_PostStack::

	PushStack = Increment the index and adding the routine address at the current index.
	PopStack  = Calling the content of current index and decrement the index.
	Content of Index == 0 is the last post routine.

	Stack index need not be larger than a UCHAR, but USHORT improves
	the data structure alignment

***********************************************************************

History :

10/02/2004: Madhuresh
Added pOSD_OsmNext for use only by OSM to keep the core dedicated field pOSD_Next
field separate. A matching field has been added in the cdadef.h as well.

09/30/2004: Madhuresh
Added pPortInfo For PAC, Probe LU etc we have to preserve this for post routine

07/19/2004 : 14:20  NAR20038  Manjunath
Added place holders to keep action sets provided by CHIM on IOB events.

11/22/2003: Madhuresh
OSDescriptor is now aligned to OSD as defined in cdadef.h.

11/19/2003: Madhuresh
Modifying TransportSpecific field in IOD to match SAS. Other cleanups.

12/23/2002: Madhuresh.
Added 32K double Buffer for Writes to RAIDs. This helps in resolving miscompare
of writes on two drives for pagefiles in nt. Pointers added in IOD.H

Oct 18, 2002: Madhuresh
LockNextRange: When background op like verify or rebuild enters a new range,
we should lock that range for the IOs. Whenever this happens
all such IODs are flagged and current RD is kept in SD for when
IOPull happens, it examins and tries to lock the range again. IOD field defined.

01/14/2002: Madhuresh.
Added a field in IOD to point to the mirror IOD. If this is ZERO we are
doing software mirroring (as used in XLateIODToIOB). Also renamed OSD_Control
to OSD_R1WriteControl (04/02/2002) for exclusive use by R1Write.

10/30/2001 : 11:13  Manjunath.
Added forward and backward pts in the IOD to facilitate dequeuing. This
optimizes performance during run time.

08/16/2001 : 12:34  Madhuresh.
Changing an unused field to create space for storing original IOD pointer
in a newly created IOD which issues Start Motor On.

07/25/2001 : 12:34  Madhuresh.
Cleanup. Removed DEBUG and WOLFPACK defines. Organized and documented
the fields.

01/24/2001 : 17:43  NAR20038  Manjunath
In IODescriptor replaced XORDevice[32] by HIM_TS_SCSI.

01/24/2001 : 02:57 PM   NAR20038  Manjunath
Removed ConfigInfoBuffer structure, deleted some fields in IODescriptor
and OSDescriptor.

12/13/2000 : 11:08 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

1. Removed #include "cache.h"
2. Removed #ifdef DATACACHEON declarations
3. Updated OSDescriptor

***********************************************************************/

#ifndef IOD_INCLUDED
#define IOD_INCLUDED

#include "sysdef.h"

#define IOPOST_MAX		8		/* Max number of call backs on post stack */
#define SENSE_MAX		32		/* Bytes in sense info */
	/*
		Values of IOD_Priority.
		Lower number means higher priority. Leave gaps for adding new priorities.
		modepage.h defines more numbers which come in modepage packets and finally
		get written into the IODs. These are mainly for the background operations
		such as Init/Verify/Rebuild.
	*/
#define PRI_StartStop	4		/* When we have to stop a IOD, priority is highest */
#define PRI_RECOVER		8		/* IODs used for recovery after a drive failed */
#define PRI_SYS_READ	16		/* Regular */
#define PRI_SYS_WRITE	16
#define PRI_GENERIC		64		/* Background or low level operations */

	/*  Values for IOD_Direction */
#define IORead			0x00	/* Same as DATA IN */
#define IOWrite			0x01	/* Same as DATA OUT */
#define IOWriteV		0x02	/* unused */
#define IONone			0x04	/* No data transfer */

	/* Values for IOD_Function */
#define STDIO			0x00	/* For Read, Write, Inquiry etc commands */
#define STOPIO			0x01	/* To stop a given IO */
#define STARTIO			0x02	/* To start a stopped IO */
#define ABORT_IO		0x04	/* To kill a given IO */
#define FLUSH_IO		0x06	/* To kill all IOs currently in the list */

#define SCSIBUSDEVICERESET	0x08	/* To issue a SCSI reset */
#define ADAPTER_FUNCTION	0X80	/* Functions specific to sequencer */

/* IOD_Function for PDE removal bits for Auto rebuild operation */
#define DELETE_PDE_OLD_MEMBER   0x10
#define DELETE_PDE_SPARE_MEMBER 0x20

	/* Values for IOD_Status */
#define ERROR_RECOVERABLE	0x00	/* Things like media recovered error. Same as no error. */
#define ERROR_READ_FAILURE	0x01	/* Read failed */
#define ERROR_WRITE_FAILURE	0x02	/* Write failed */

	/* Values for bIOD_fPACRequest flag */
#define PAC_REQUEST			0x01	/* Set this bit for issuing a Protocol Auto Config */

	/* Max number of IODs allowed to be inserted in the middle of the queue. */
#define MAX_INSERT			200		/* To implement proxy */
#define IODS_PER_R1WRITE	2		/* Of course. */

/****************************************************************************
		The IOD.

		This structure contains fields to perform or queue an I/O on the
		Physical side.
****************************************************************************/
struct  _IODescriptor
{
	HIM_IOB			IOD_IOB;	/* Embed an IOB */
	UCHAR			IOD_Junk1   [(4- sizeof(HIM_IOB) % 4 )]; /* Align to ULONG */

	pIODescriptor	pIOD_Next; /* ptr to next iod in the list	*/
	pIODescriptor	pIOD_Back; /* ptr to previous iod in the list */
	pIODescriptor	pIOD_SeqMirror ; /* passive iod, if in sequencer mirroring is supported. */
	pIODescriptor	pIOD_ParentOfRetry;	/* Holds original IOD from which MotorOn IOD was created */
	pIODescriptor	pIOD_SpReq_Next; /* To queue special operation IODs such as UpdateCheckPoint, Abort, Reset */
	pIODescriptor	pIOD_RSReq_Next;/* To queue the meta data read/write IODs for re-entrancy. */
    pIODescriptor	pIOD_Tmf_Next;  /* This chains IODs we may need to be TMF due to IO timeout*/

	USHORT			IOD_Priority;	/* See priorities above - lower number is higher priority */
	USHORT			IOD_Function;	/* See the list of functions above */
	FPTR			pIOD_Submit;	/* To remember the called entry point. - Unused */
	PVOID			pIOD_IOParent;	/* The pOSD who created this IOD. */
	pPhysDevice		pIOD_PhysParent;/* Physical Device that this IOD works on. */
	pSysDevice		pIOD_SysParent;	/* SysDevice on which this IOD works. */

	BOOL			bIOD_fRecoveryIO;	/* This is an IOD started for some recovery operation */
	BOOL			bIOD_fIOCtlRequest:8;/* This IOD is for a internal IOCTL call (maybe from iomgr) */
	BOOL			bIOD_fPACRequest:8;	/* Protocol Auto Config IOD */
	UINT8			IOD_BitFlags;
#define CDA_IOD_FLAG_MEH_RECOVERY_IO			0x01
#define CDA_IOD_FLAG_METADATA_MEH_RECOVERY_IO	0x02

	BOOL			bIOD_AtHIM;			/* The IOD is owned by HIM at this time. */
	BOOL			bIOD_AbortIsPending;/* Upper layers have requested to abort this IOD. */
	BOOL			bOverlapsVR ;	/* Verify/Rebuild intends to enter a range but finds this IOD is */
									/* is already in some queue for that range. */

	USHORT			cIOD_Ticks;		/* Max ticks for each retry. */
	USHORT			cIOD_TCount;	/* If zero, this IOD has timed out and no response came from CHIM. */
	USHORT			cIOD_Retry;		/* If zero, do not try this IOD again. */
	ULONG			IOD_AccessBlock;/* Zero sector relative address for this drive. */
	ULONG			IOD_StartBlock;	/* Address relative to the sys device */
	ULONG			IOD_cBlocks;	/* Number of blocks. */
	ULONG			IOD_RequestedBlocks;


	UCHAR			IOD_CDB[MAX_CDB_LEN];
	USHORT			IOD_cCDB;		/* Number of bytes in CDB */
	UCHAR			IOD_Direction;	/* Data Transfer direction (values are defined above) */
	UCHAR			IOD_Status;		/* How was the IO executed. non-zero means errors. */
	SCSISenseData	IOD_Sense;		/* The sense bytes picked from the drive. */
	USHORT			IOD_SenseLen;	/* How many sense bytes were picked. */

	USHORT			iIOD_PostStack;	/* See comments above (search for this field name) */
	FPTR			pIOD_PostStack[IOPOST_MAX];/* See comments above */

	ULONG			IOD_TimeStamp;	/* IOSort() uses this to limit the number of inserts */
	FPTR			pIOD_QueuePost;	/* routine to receive IOD */
	BOOL			bIOD_OverrideSuspend; /* we suspended to allow this to go through alone */
	//UCHAR			IOD_Padding ;	/* The BOOL above uses only two bytes. ULONG aligned now. */
    UCHAR			IOD_DiskFuncModeFlag ; /* DiskFuncModeFlag moved here instead of a global */
                                           /* defined in ADAPTER_DEV_EXT */
    UCHAR           IOD_RetryTimeout; /* IOD_RetryTimeout is used to delay the posting of IOD to HIM */

	FPTR			pIOD_Retry;		/* The routine to use for retry such special operations. */
	ULONG			AbortThreshold; /* Abort command 3 times maximum before doing a reset. */

#if	ALLOW_PROXY
	pIODescriptor	DPTR	   pIOD_Proxy;/* For combining multiple IODs (Proxy) */
	USHORT			IOD_MySGCount;	/* For combining multiple IODs (Proxy) */
	USHORT			IOD_AllowProxy;	/* Combine multiple IODs (Proxy)? */

#endif /* ALLOW_PROXY */
    union {
#ifdef SCO_DDI8
	HIM_TS_SAS		_TS_SAS;
    	HIM_TS_SATA     	_TS_SATA;
	} _TS_union;
#define TransportSpecific	_TS_union._TS_SAS
#define TransportSpecificSATA	_TS_union._TS_SATA

#else 	/* SCO_DDI8 */
	HIM_TS_SAS		TransportSpecific;
    HIM_TS_SATA     TransportSpecificSATA;
	};
#endif 	/* SCO_DDI8 */

	BOOL            IOB_Post;
    PVOID           pIOD_Special_Mem;    /* tt used to store special memory required by CHIM */
	PVOID           pPortInfo ;		/* For PAC, Probe LU etc we have to preserve this for post routine */

    UCHAR           IOD_QueueTag;
	UCHAR			IOD_CustomFlag;	/* TDE - Indicates OSM processing I/Os with busy Logic */
    UCHAR           IOD_NoDispatchOnRetry;  /* This flag is used to avoid dispatching IO */
                                            /* to CHIM immediately */
    ULONG   SavedActionSet; /* original action set when port event is generated */
    ULONG   WorkingActionSet; /* working action set as we progress through port event handling */
    PVOID   SavedEventContext; /* saved CHIM context for the event @TR*/

    DATA_N_PTR	BuffVirtAddr;  /* Some OS like Netware will require the virtual address for 
                                  pIOD_SGTable->SGList[0].SGLBufAddr to be saved since it can not 
                                  get the virtual on it's own
                                */

    /* The S/G list always be at the end since sg list will be assigned dynamically */
	pSGTable		pIOD_SGTable;	/* Pointer to the SG Table below. Meant for CHIM to DMA the table itself */
	SGTable			IOD_SGTable;	/* 16 bytes. Has first SG entry, rest follow */

};

/**************************************************************
	The new OSDescriptor.

	Aligned to OSD as defined in cdadef.h for new interface.

**************************************************************/
typedef struct  _OSDescriptor
{
    pOSDescriptor   pOSD_Next;
	pOSDescriptor	pOSD_OsmBack; /* ptr to previous osd in the list */
    pOSDescriptor   pOSD_OsmNext;/* Used only by OSM */

	pSysDevice		pOSD_SysDevice;
    UINT8           OSD_Direction ;
    UINT8           OSD_RetryCount ;
    UINT8           OSD_SenseLen ;
    UINT8           OSD_cCDB ;
    UINT8           OSD_CDB[ CDA_MAX_CDB_LENGTH ] ;
    SCSISenseData   OSD_Sense;		/* The Sense bytes from drive */
    void            *pOSD_Sense;        /* OSM will use this pointer to access sense data */
                                        /* CDA only modify this pointer for removable media */
    SGTable         OSD_SGTable ;

	UINT32			OSD_DataTransferLength ;
	pSGTable		pOSD_SGTable;	/* Pointer to the SG Table below. Meant for CHIM to DMA the table itself. */
    DATA_N_PTR      *OSD_pDataBuffer ;
    FNPTR           pOSD_Post ;
    void           *OSD_DeviceExtension ;
    void           *pOSRequest ;
	/*
		Fields below this line are for internal use of CDA RAID engine.
		OSM or HCI
	*/
	FNPTR           pOSD_Retry ;
	ULONG			OSD_StartBlock ;    /* IO starting block */
	ULONG			cOSD_Blocks ;       /* Total block count.*/

	UCHAR			OSD_R1WriteControl ;/* Now used for Raid1 Write only.*/
#define		ONE_MEMBER_WRITTEN		0x04
#define     CDA_OSD_FLAG_MEH_RECOVERY_IO    0x01    

	UCHAR			cOSD_Reference ;    /* Call post when 0 */
	UCHAR			iMember ;           /* Member number - used only by IssueStartStopUnitToPairs */    
	UCHAR			OSD_Priority ;      /* Priority for array management */	

	UINT16          OSD_Status ;
	USHORT			cOSD_TCount ;       /* Number of ticks for operation to complete. */
	
	USHORT			OSD_OurTimeout;
	UCHAR			OSD_CustomFlag;		/* TDE - Indicates OSM processing I/Os with busy Logic */
	UCHAR			bOSD_fIOCtlRequest;

	UCHAR           OSD_QueueTag;
	UCHAR			SendStopUnitCmd;	/* Send stop unit command to device when shutdowning system */
	UCHAR			OSD_Padding[2];
	

	//
	// Following fields are additional fields for OSDecriptor only.
	//
	pIODescriptor	pOSD_IODChain;		/* TDE - Needed to keep track of all IODs allocated */
										/* for this OSD in case TMF is issued by OSM.   	*/
	BYTE			MemberIOStat;
#define		RESET_MEMBER_IO_STAT	-1


} OSDescriptor ;


#endif
