/*
 * Copyright  2005 Adaptec, Inc.  All rights reserved.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that i) redistribution of source code must retain the above copyright
 * notices, this list of conditions and the following disclaimer; ii)
 * redistribution in binary form must reproduce  the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution and iii) Adaptecs
 * name may not be used to endorse or promote products derived from this
 * software without specific written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY ADAPTEC AS IS AND ADAPTEC DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THE SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT
 * SHALL ADAPTEC, INC. BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR
 * FOR SPECIAL, INDIRECT, CONSEQUENTIAL OR INCIDENTAL DAMAGES OR ANY OTHER
 * DAMAGES WHATSOEVER, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
 * LIABILITY, ARISING OUT OF IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

/**********************************************************************
Filename: modepage.h

File Description
Definition of mode pages for use at runtime. The definition of these
structures leverages heavily from the stored configuration information.

***********************************************************************

History :

10/18/05: cha23416 Naveen Chandrasekaran
Add SCSI Error Recovery Mode page data structure

10/08/03: Madhuresh
MAX_PER_BUS is defined in sysdef.h. Avoid duplication here. The number
goes up to 128 in Razor.

04/18/2003 10:45 NAR20038 Manjunath
Changed PRIORITY_MEDIUM from 32 to 16. This now gets aligned with READ_PRI
and WRITE_PRIORITY. This way in IOSort(), we will have opportunity to sort
requests and apply elevater algorithm.

In AdatperTypePage, use one of the Padding bytes for storing number of ports
for SATA controller.

Replaced MAXARRAYMEMBERS to 5. We need to do this to align with U320 
Channel release and to have single binary of IOMgr for both SCSI and SATA.

Replaced unused Diagnostics field in controller type page with board 
serial number.

Sept 19, 2002: Madhuresh
A free field has been converted to CT_ReservedInEnd which means the
Number of blocks used for Meta Data in End of disk and an old unused
field CT_RelativeIndex has been recycled for CT_ReservedInBeginning.
As part of DST implementation but present in all releases.

07/11/2002 : 13:30 BUR11729 Farrukh
Added more capabalities to export to IO Manager for IBIS Support in
ControllerFuncPage.

01/31/2002 : 12:28 PM   BUR11729 Farrukh
Changes made to export capabilities to IO Manager for IBIS support.
Some defines moved to sysdef.h that are relevent to marketing specs.

01/31/2001 : 10:34 PM   NAR20038  Manjunath
Added few cache related defines so that IOMgr would compile. Later we 
can remove them once we clean up IOMgr.

12/13/2000 : 11:19 AM   NAR20038  Manjunath

Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

** modepage.h

1. Removed PAGE_CACHE_DEF
2. Removed CacheDefinitionPage
3. Removed CAP_PROFILING
4. Removed CAP_CACHEING
5. Removed CAP_RESERVED1_FOR_IOM
6. Removed CF_CacheSize from ControllerFuncPage

***********************************************************************/

#ifndef MODEPAGE_INCLUDED
#define MODEPAGE_INCLUDED

#include "sysdef.h"

#define IS_IO		0
#define IS_MEM		1

/*********************************************************************

	C O N T R O L L E R   P A G E S

*********************************************************************/


/* Define the qualifiers that shows if the device below	*/
/* is a controller or some device.							*/

#define	CONTROLLER_DEVICE		0
#define	STORAGE_DEVICE			1
#define	ADAPTER_DEVICE			2

#define	CONTROLLER_VERSION		2
#define	CONTROLLER_REVISION		0


typedef struct _DeviceTypePage		/* structure size is 56 (0x3C) bytes */
{
/* 0x0000 */	LDTree	 	DT_Path;
/* 0x0014 */	ULONG		DT_UniqueId;
/* 0x0018 */	UCHAR		DT_AccessType;	/* LSingle, RSingle......*/
/* 0x0019 */	UCHAR		DT_StorageType; /* SCSI Device type */
/* 0x001A */	USHORT		DT_PhysDriveIndex;
/* 0x001C */	USHORT		DT_Status;		/* SUCCESS or FAILURE:This reflects ArrayState */
/* 0x001E */	USHORT		DT_SubStatus; 	/* This reflects ArraySubState */
/* 0x0020 */	USHORT		DT_Padding;		/* Unused */
/* 0x0022 */	USHORT		DT_Convertible; /* Non zero if convertible 0 to 5 */

	/* These fields are applicable only if the device reports to be a disk.	*/
	
/* 0x0024 */	ULONG		DT_nCyl;		/* Reported cylinders */
/* 0x0028 */	ULONG		DT_nHeads;		/* Reported heads */
/* 0x002C */	ULONG		DT_nSPT;   		/* Reported sectors per track */
/* 0x0030 */	ULONG		DT_nBlocks;		/* Total addressable block count */

	/* We note if this device has beeb reported previously. This allows		*/
	/* callers to quickly note any device found after the last report of	*/
	/* configuration.  If newly found this value is non zero.				*/

/* 0x0034 */	USHORT	 	DT_NewlyFound;

	/* These fields provide a mapping for the system device to be reported	*/
	/* at some address. The server does not depend on this information,		*/
	/* rather the LDTree is used to target device. The client uses this 	*/
	/* information as a guideline for the mapping of devices to some set of	*/
	/* busses. There is no guaratee that the client will preserve the 		*/
	/* information of bus,id and LUN.										*/
	
/* 0x0036 */	USHORT	 	DT_ReportedBus;
/* 0x0038 */	USHORT	 	DT_ReportedId;
/* 0x003A */	USHORT		DT_ReportedLUN;

}DeviceTypePage, DPTR pDeviceTypePage;

/***********************************************************************
	MODE PAGES for CONTROLLER DEVICE

	PAGE_LD_DEF defines all logical devices attached below.
	PAGE_CACHE_DEF defines the cacheing at the controller.
	PAGE_STORAGE_DEF defines ALL storage devices below this device.
	PAGE_CONTROLLER_FUNC defines what atomatic features are allowed.
***********************************************************************/


#define	PAGE_LD_DEF			0x00

#define	TYPE_VIKING			0
#define	TYPE_ATHENA			1
#define	TYPE_THOR			2
#define	TYPE_ZEUS			3

typedef struct _ControllerTypePage	/* structure size is 60 (0x3C) bytes */
{
/* 0x0000 */	LDTree		CT_Path;
/* 0x0014 */	ULONG		CT_UniqueId;		/* Used for conflict resolution */
/* 0x0018 */	USHORT		CT_ControllerType;
/* 0x001A */	USHORT		CT_ControllerVersion;
/* 0x001C */	USHORT		CT_ControllerRevision;
/* 0x001E */	UCHAR		CT_MaxAdapter;
/* 0x001F */	UCHAR		CT_Padding ;		/* TDE - merged code */				
/* 0x0020 */	UCHAR		CT_PCIBus;
/* 0x0021 */	UCHAR		CT_PCIDevice;
/* 0x0022 */	UCHAR		CT_MemOrIO;
/* 0x0023 */	UCHAR		CT_ReservedInBeginning ;	/* CT_RelativeIndex;*/
/* 0x0024 */	ULONG		CT_BaseAddr;
/* 0x0028 */	ULONG		CT_MemBaseAddr;		/* Base for memory mapped access */
/* 0x002C */	ULONG		CT_ReservedInEnd ;	/* TDE - merged code - */
												/* Number of blocks used for Meta Data in End of disk */
/* 0x0030 */	USHORT		CT_PCIType;
/* 0x0032 */	USHORT		CT_PCIVendorId;		/* PCI Vendor */
/* 0x0034 */	USHORT		CT_PCIDeviceId;		/* PCI Device */
/* 0x0036 */	UCHAR		CT_PCIDevFunct;		/* PCI Device & Function #s from Find_PCI_Device */
/* 0x0037 */	UCHAR		CT_IntLine;			/* Level for the interrupt for this card */
/* 0x0038 */	ULONG		CT_SerialNumber;
} ControllerTypePage, DPTR pControllerTypePage;

/* Definition of AT_Status */
#define CDA_ADAPTER_STATUS_ONLINE				  0x00
#define CDA_ADAPTER_STATUS_OFFLINE				  0x01
#define CDA_ADAPTER_STATUS_RECOVERED_CONFIG_ERROR 0x02

typedef struct	_AdapterTypePage 	/* structure size is 56 (0x38) bytes */
{
/* 0x0000 */	LDTree		AT_Path;
/* 0x0014 */	ULONG		AT_BaseAddrHigh;	/* High part of 64bit I/O or memory mapped base logical address */
/* 0x0018 */	ULONG		AT_BaseAddrLow;		/* Low part of 64bit I/O or memory mapped base logical address  */
///* 0x001C */	ULONG		AT_RomAddr;			/* Loc of expansion rom address	*/
/* SAS support */
/* 0x001C */    UCHAR       AT_Status;          /* See Definitions above */
/* 0x001D */    UCHAR       AT_NumSASPorts;     /* Number of discovered SAS ports */
/* 0x001E */    UCHAR       AT_NumSASPhys;      /* Number of discovered SAS PHYs */
/* 0x001F */    UCHAR       AT_Future1;         /* */
	
/* 0x0020 */	USHORT		AT_RelativeIndex;	/* Controller relative bus index */
/* 0x0022 */	USHORT		AT_PCIType;
/* 0x0024 */	USHORT		AT_PCIVendorId;		/* PCI Vendor */
/* 0x0026 */	USHORT		AT_PCIDeviceId;		/* PCI Device */
/* 0x0028 */	UCHAR		AT_PCIDevFunct;		/* PCI Device & Function #s from Find_PCI_Device */
/* 0x0029 */	UCHAR		AT_IntLine;			/* Level for the interrupt for this card */
/* 0x002A */	UCHAR		AT_PCIBus;			/* PCI Bus Number */
/* 0x002B */	UCHAR		AT_PCIDevice;		/* PCI Divice Number */
/* 0x002C */	UCHAR		AT_MemOrIO;
/* 0x002D */	UCHAR		AT_NumPorts;
/* 0x002E */	UCHAR		Padding[2];			/* padding for dword alignment */
/* 0x0030 */	USHORT		AT_Clk;				/* Setting for input clock */
/* 0x0032 */	USHORT		AT_PhysInterface;	/* Type of bus, PHYS_SCSI2, PHYS_SSA, etc */
/* 0x0034 */	ULONG		AT_MaxTargets;
/* 0x0038 */	ULONG		AT_MaxLunsPerTarget;
/* 0x003C */	ULONG		AT_IOBusWidth;		/* wide or narrow */
/* 0x0040 */	ULONG		AT_IOBusSpeed;		/* 320, 160 etc. */
/* 0x0044 */	USHORT		AT_MaxDevices;		/* Max devices on this type of medium */
/* 0x0046 */	USHORT		AT_CrntDevices;		/* Num devices known to be attached	*/
} AdapterTypePage, DPTR pAdapterTypePage;

typedef struct _UnitTypePage 			/* structure size is 64 (0x40) bytes */
{
/* 0x0000 */	USHORT		UT_Qualifier;	/* Type of page	*/
/* 0x0002 */	USHORT		Padding;		/* padding to dword alignment */
/* 0x0004 */	union _UT_UnitDef
				{
					ControllerTypePage	CDef;
					AdapterTypePage		ADef;
					DeviceTypePage		DDef;
				} UT_UnitDef;
} UnitTypePage, DPTR pUnitTypePage;
		
typedef struct _ControllerConfigPage	/* structure size is 68 (0x44) bytes */
{
/* 0x0000 */	USHORT			cCC_Units;
/* 0x0002 */	USHORT			Padding;		/* padding to dword alignment */
/* 0x0004 */	UnitTypePage	CC_Units[1];
} ControllerConfigPage, DPTR pControllerConfigPage;

#define PAGE_STORAGE_DEF		0x02

typedef struct _StorageDefPage		/* structure size is 60 (0x3C) bytes */
{
/* 0x0000 */	USHORT			cSD_Units;
/* 0x0002 */	USHORT			Padding;		/* padding to dword alignment */
/* 0x0004 */	DeviceTypePage	SD_Path[1];
} StorageDefPage, DPTR pStorageDefPage;


#define PAGE_CONTROLLER_FUNC	0x03


typedef struct	_ControllerFuncPage /* structure size is 24 (0x18) bytes */
{
/* 0x0000 */	ULONG		CF_Capabilities;	  /* BIT map for feature set supported for management such as */
    											  /* Dedicated Spare, Pool Spare, Logical Arrays, Auto verify *												* and Auto Fix.											*/
/* 0x0004 */	ULONG		CF_RaidDefinitions;   /* Define Array types supported. Defined as a bit map in sysdef.h	*/
/* 0x0008 */	UCHAR		CF_cPhysArrays; 	  /* num phys arrays that may be grouped into logical arrays */
/* 0x0009 */	UCHAR		CF_cLogicalArrays; 	  /* num arrays on a drive */
/* 0x000A */	USHORT		CF_MaxDstripe;		  /* data stripe size in kbytes*/
/* 0x000C */	USHORT		CF_MinDstripe;
/* 0x000E */	USHORT		CF_MaxPstripe;		  /* max parity stripe size in kbytes */
/* 0x0010 */	USHORT		CF_MinPstripe;		  /* min in kbytes 				*/
/* 0x0012 */	USHORT		CF_DisableBiosReport; /* Do not report units at bios startup */
/* 0x0014 */	USHORT		CF_MaxAutoResume;	  /* Maximum value allow to wait before resume */
/* 0x0015 */	UCHAR		CF_QuickInitSupport;  /* Can OS access the RAID while it is partially intialized? */
/* 0x0016 */	UCHAR		CF_ContainersSupport;
/* 0x0017 */	UCHAR		CF_MorphingSupportMask;
/* 0x0018 */	UCHAR		CF_OnlineCapExpantionSupport;
/* 0x0019 */	UCHAR		CF_ScrubSupport;      /* High end RAIDs call verify as scrubbing. */
/* 0x001A */	UCHAR		CF_CrossChannelRAIDSupport;
/* 0x001B */	UCHAR		CF_Clustering;
/* 0x001C */	UCHAR		CF_MaxSparesPerRAID;  /* Dedicated spares per RAID */
/* 0x001D */	UCHAR		CF_MaxSparesInPool;	  /* Max spares in Pool */
/* 0x001E */	UCHAR		CF_MaxMemberPerRAID0;
/* 0x001F */	UCHAR		CF_MinMemberPerRAID0;
/* 0x0020 */	UCHAR		CF_MaxMemberPerRAID1;
/* 0x0021 */	UCHAR		CF_MinMemberPerRAID1;
/* 0x0022 */	UCHAR		CF_MaxMemberPerRAID3;
/* 0x0023 */	UCHAR		CF_MinMemberPerRAID3;
/* 0x0024 */	UCHAR		CF_MaxMemberPerRAID4;
/* 0x0025 */	UCHAR		CF_MinMemberPerRAID4;
/* 0x0026 */	UCHAR		CF_MaxMemberPerRAID5;
/* 0x0027 */	UCHAR		CF_MinMemberPerRAID5;
/* 0x0028 */	UCHAR		CF_MaxMemberPerRAID10;
/* 0x0029 */	UCHAR		CF_MinMemberPerRAID10;
/* 0x002A */	UCHAR		CF_MaxMemberPerRAID50;
/* 0x002B */	UCHAR		CF_MinMemberPerRAID50;
/* 0x002C */	UCHAR		CF_MaxRAIDSPerBus;
/* 0x002D */	UCHAR		CF_MaxBusPerSystem;
/* 0x002E */	UCHAR		CF_MaxRAIDSPerSystem;
/* 0x002F */	UCHAR		CF_MaxRAID0;
/* 0x0030 */	UCHAR		CF_MaxRAID1;
/* 0x0031 */	UCHAR		CF_MaxRAID3;
/* 0x0032 */	UCHAR		CF_MaxRAID4;
/* 0x0033 */	UCHAR		CF_MaxRAID5;
/* 0x0034 */	UCHAR		CF_MaxRAID10;
/* 0x0035 */	UCHAR		CF_MaxRAID50;
/* 0x0036 */	UCHAR		CF_FormatWithCreation;
/* 0x0037 */	UCHAR		CF_BatterySupport;
/* 0x0038 */	UCHAR		CF_ProfilingSupport;
/* 0x0039 */	UCHAR		CF_AutoReconstructSupport;
/* 0x003A */	UCHAR		CF_SupportBuildAtCreate;
/* 0x003B */	UCHAR		CF_maxRAIDVolumeMembers;
/* 0x003C */	UCHAR 		CF_DeviceSupportClear;		/* Define device capabilities. Defined as a bit map in IBIS*/
/* 0x003D */	UCHAR		CF_ZeroInitOSArray;			/* Zero the array with a partition */
/* 0x003F */	UCHAR		CF_ConfigureSingleDrive;	/* Driver supports configuring single drives with meta data */
/* 0x0040 */	UCHAR		CF_ChangeWriteCacheSettings;	/* Driver supports write cache settings */
/* 0x0041 */	UCHAR		CF_DDFDriveSizeCoercion;	/* DDF drive coersion algorithm */
/* 0x0042 */    USHORT		CF_CacheSize;
/* 0x0044 */    UCHAR       CF_MaxSmpPages;             /* How many elements are there in array StoredMemberPage[] in internal meta data view */
/* 0x0045 */    UCHAR       CF_BiosFlashingSupport;     /* IBM special but this will stay on.*/
/* 0x0046 */	UCHAR		CF_MaxRAIDMembers;          /* Max members in any RAID multiplied by Max RAIDs connected to the chip */
/* 0x0047 */    UCHAR       CF_ForcedDownADrive;
/* 0x0048 */    UCHAR       CF_ReactivateArray;
/* 0x0049 */    UCHAR       CF_cCCUnitsPerPage;         /* Number of devices that will reported on each page */
/* 0x004A */    UCHAR       CF_MaxConfiguredSingleDrives;
/* 0x004B */	UCHAR		CF_MarkBootable; //@Maram	
/* 0x004C */	UCHAR		CF_NameSimpleVolume;    

/* 0x004D */    UCHAR        Reserved[52];

 

 

 



} ControllerFuncPage, DPTR pControllerFuncPage;



#define PAGE_CONTROLLER_CODE_MGT 0x04

/* This ioctl call  passes a 4 byte address of host system RAM: a pointer	*/
/* to a buffer in the system RAM which will be used to do Block Move in	*/
/* Sabre space.															*/
#define PAGE_SABRE_BLOCKMOVE	0x06

#define WRITE_CODE_IMAGE		0x00

#define REVERT_CODE_IMAGE		0x01

#define RETURN_CRNT_CODE		0x10
#define RETURN_CRNT_BIOS		0x20
#define RETURN_PREV_BIOS		0x40

typedef struct _ControllerCodeMgtPage 	/* structure size is 4 (0x04) bytes */
{
/* 0x0000 */	ULONG			CCM_Func;
} ControllerCodeMgtPage, DPTR pControllerCodeMgtPage;

/* This is an ioctl shutdown.  This is generated a shutdown event is	*/
/* about to happen.  Before a system system, the driver needs to set	*/
/* a flag in the ReservedSectorLayout to indicate a proper shutdown.	*/
#define PAGE_SHUTDOWN			0x05
#define PAGE_STARTUP			0x07

#define PAGE_CONTROLLER_FAULT	0x08
#define PAGE_KNOWN_TO_OS		0x09
#define PAGE_CONTR_SETTINGS   	0x0A
#define PAGE_TRUE_ID			0x0B

/* The structure that defines PAGE_CONTROLLER_FAULT modepage is Athena		*/
/* specific.  A similar but larger size structure has already been defined */
/* in ft_defs.h in the Athena side of the code. The following structures	*/
/* are redefined to fit the 32 bytes size in the NVRAM EventProfile area.	*/
typedef struct NVRAM_i960_information_struct /* structure size is 16 (0x10) bytes */
{
/* 0x0000 */    UCHAR		fault_class;
/* 0x0001 */    UCHAR		pad1;
/* 0x0002 */    UCHAR		pad2;
/* 0x0003 */    UCHAR		pad3;
/* 0x0004 */	 ULONG DPTR fault_IP;
/* 0x0008 */	 ULONG		PC_reg;
/* 0x000C */	 ULONG		AC_reg;
} NVRAM_I960_INFORMATION;


typedef struct NVRAM_dram_information_struct /* structure size is 16 (0x10) bytes */
{
			    /* below are registers for the plx dma operation */
/* 0x0000 */    ULONG		dma_status_reg;
/* 0x0004 */    ULONG		dma0_mode_reg;
/* 0x0008 */    ULONG		dma1_mode_reg;
			    /* below is one of the registers for the I960 dram operation */
/* 0x000C */    ULONG	IP_reg;
} NVRAM_DRAM_INFORMATION;


typedef struct NVRAM_v3_information_struct /* structure size is 16 (0x10) bytes */
{
			    /* below are registers for the v3 operation */
/* 0x0000 */    USHORT		v3_int_status_reg;
/* 0x0002 */    USHORT 	v3_dma0_status_reg;
/* 0x0004 */    USHORT 	v3_dma1_status_reg;
/* 0x0006 */    USHORT 	pad;
/* 0x0008 */    ULONG  	PC_reg;
/* 0x000C */    ULONG  	IP_reg;
} NVRAM_V3_INFORMATION;


typedef struct NVRAM_plx_information_struct /* structure size is 16 (0x10) bytes */
{
			    /* below are registers for the PLX operation */
/* 0x0000 */    USHORT		plx_vendorID_reg;
/* 0x0002 */    USHORT		plx_deviceID_reg;
/* 0x0004 */    USHORT		plx_pci_cmd_reg;
/* 0x0006 */    USHORT		plx_pci_sts_reg;
/* 0x0008 */    ULONG  	PC_reg;
/* 0x000C */    ULONG  	IP_reg;
} NVRAM_PLX_INFORMATION;


typedef struct NVRAM_aic_information_struct /* structure size is 4 (0x04) bytes */
{
/* 0x0000 */    UCHAR	cfp_port;      /* port: Lance0, Lance1 or Sabre2 */
				/* below are registers for the plx dma operation */
/* 0x0001 */    UCHAR	hard_error_reg;
/* 0x0002 */    UCHAR	pad1;
/* 0x0003 */    UCHAR	pad2;
} NVRAM_AIC_INFORMATION;



typedef struct _ControllerFaultPage /* structure size is 32 (0x20) bytes */
{
/* 0x0000 */    UCHAR exception_type;
/* 0x0001 */    UCHAR exception_subtype;
			    /* TRUE indicates success, FALSE indicates failure */
/* 0x0002 */    UCHAR retryOK;
			    /* by exception_class to distinguish I960 fault, DRAM parity error or V3 error */
/* 0x0003 */    UCHAR exception_class;
			    /* indicate that how many retry have been done when suceed or fail */
/* 0x0004 */    ULONG  retry_count;
/* 0x0008 */    ULONG  athena_status;
			    /* pad 4 byte to make long word boundary */
/* 0x000C */    ULONG  padding;     
			    /* access the detailed informations about DRAM parity error information */
/* 0x0010 */    union 
				{
			        NVRAM_I960_INFORMATION   i960_info;
			        NVRAM_DRAM_INFORMATION   dram_info;
			        NVRAM_V3_INFORMATION     v3_info;
			        NVRAM_PLX_INFORMATION    plx_info;
			        NVRAM_AIC_INFORMATION    aic_info;
			    } info;
} ControllerFaultPage, DPTR pControllerFaultPage;


/***********************************************************************
	MODE PAGE FOR AN ADAPTER NVRAM SETTINGS
	PAGE_CONTR_SETTINGS accesses this structure.
***********************************************************************/
typedef struct _ControllerSettings	/* structure size is 4 bytes */
{
/* 0x0000 */	ULONG	Int13DeviceReport:1;	/* Zero disables reporting */
				ULONG	padding:31; 	
} ControllerSettings, DPTR pControllerSettings;


/***********************************************************************
	MODE PAGES FOR AN ADAPTER DEVICE
	PAGE_ADAPTER_DEF defines all adapters reported directly beneath.
***********************************************************************/

#define PAGE_ADAPTER_DEF 	0x00

typedef struct _AdapterDefPage 	/* structure size is 60 (0x3C) bytes */
{
/* 0x0000 */	USHORT			cAD_Units;
/* 0x0002 */	USHORT			Padding;		/* padding to dword alignment */
/* 0x0004 */	AdapterTypePage	AD_Def[1];
} AdapterDefPage, DPTR pAdapterDefPage;

/* Changing mode pages and nvram definition 							*/
/*   - AdapterDeviceSettings is ATHENA specific 						*/
/*    Viking HIM will set SCSIOptions from eeprom, driver level		*/
/*    no longer changes SCSIOptions directly for Viking or Athena.		*/
/*   - Be careful that BIT MASKS might not work the same between		*/
/*     Athena and Viking												*/
/*      --------------------------------------------------------------	*/
/*      SYNC_RATE     ACK Assertion     ACK Deassertion     Rate (MHz)	*/
/*      --------------------------------------------------------------	*/
/*        000            50 nsec            50 nsec            10.0	*/
/*        001            50 nsec            75 nsec             8.0	*/
/*        010            50 nsec           100 nsec             6.7	*/
/*        011            50 nsec           125 nsec             5.7	*/
/*        100           100 nsec           100 nsec             5.0	*/
/*        101           100 nsec           125 nsec             4.4	*/
/*        110           100 nsec           150 nsec             4.0	*/
/*        111           100 nsec           175 nsec             3.6	*/
/*																		*/
/*Defines for PerDevice:												*/
/* Below definition left as original, add a 2nd byte.					*/
/* note the bit definitions below are NOT those used in the Viking HIM */
/* SG:  Moved Tag Command queue size to its own byte; added Ultra */
#define UNITSTART_BIT	0x0080 		/* bit 7 Send start unit if set	*/
#define PARITY_BIT		0x0040		/* bit 6 not used currently */
#define WIDE_BIT	   	0x0020		/* bit 5 Wide if set. */
#define DR_BIT			0x0010		/* bit 4 Use disconnect if set.	*/

#define ULTRA_BIT		0x0008		/* Must be kept with Sync Mask	*/
#define SYNC_MASK		0x0007	   	/* Uses low 3 bits for sync speed.*/
#define ULTRA_MASK		0x000F		/* Sync speed with ultra bit */

/* Below definitions are new for ArSelect byte 1 						*/
/* SG: Sync bit is moved from first byte; expansion bits must be zero	*/
#define IN_BIOS_SCAN     0x0100     /* bit 0 use drive in bios scan	*/
#define TAG_CMD_ENABLE   0x0200     /* bit 1 2 tag cmd queue enable	*/
#define SYNC_BIT		  0x0400	 /* bit 3 Allow Sync xfer if set.	*/

/*	Defines for Term. */
#define	TERMINATION_LOW		0x01
#define	TERMINATION_HIGH	0x02

#define PAGE_ADAPTER_INTERFACE_SETTINGS 0x02

/* Change the bits stored for each channel below:				*/
/* - now using 2 bytes  16 bits = USHORT for each device		*/
/* - change the per channel definitions to bitmaps over 16 bits*/
/*   save space because we will store more information			*/
typedef struct _DeviceDef					/* structure size is 4 (0x04) bytes */
{
/* 0x0000 */	USHORT		DevBits;
/* 0x0002 */	UCHAR		TagQueue;
/* 0x0003 */	UCHAR		Padding;
} PerDev;

/*#define MAX_PER_BUS 16*/
typedef struct _AdapterDeviceSettings		/* structure size is 68 (0x44) bytes */
{
/* 0x0000 */	PerDev         	PerDevice[MAX_PER_BUS];
/* 0x0040 */	ULONG    		InitrID:4;			   	/* Initiator SCSI ID */
				ULONG	      	AutoTermEnable:1;    	/* Controller Auto SCSI termination */
				ULONG			TermEnable:2;			/* Viking termination */
				ULONG	       	ScsiParity:1;		   	/* Parity enabled if set to 1 */
				ULONG 	    	UltraEnable:1;			/* Ultra enabled if set to 1 */
				ULONG	       	PwrOnScsiReset:1;	 	/* Reset on Power on enabled if set */
				ULONG 			Padding:22;		    	/* DWORD alignment */
} AdapterDeviceSettings, DPTR pAdapterDeviceSettings;

/* New mode page for results of boot scanning */
#define PAGE_ADAPTER_SCAN_RESULTS 		0x03
#define NEG_SPEED_MASK					0x0007      /* device response speed */
#define WIDE_FOUND						0x0008      /* device responds using Wide */
#define ULTRA_FOUND					0x0010      /* device responds using Ultra */

#define BOOT_STATE_MASK        		0xF000      /* basic boot location    */
#define BOOT_ADAPTER_MASK      		0x0F00      /* Adapter (Channel) used */
#define BOOT_ADAPTER_DEVICE_MASK		0x00F0      /* device scsi id */
#define BOOT_ADAPTER_DEVICE_LUN_MASK	0x000F      /* device lun not used */


typedef struct _AdapterDeviceScanResults		/* structure size is 24 (0x18) bytes */
{
/* 0x0000 */	UCHAR         PerDeviceResults[MAX_PER_BUS];
/* 0x0010 */   ULONG         BootProgress;
/* 0x0014 */   ULONG         BootProgressDetails;
} AdapterDeviceScanResults, DPTR pAdapterDeviceScanResults;

#define SMART_DRIVE				4 // this value was chosen at the request of the CIO folks

/* Definition for SD_ExtendedAttributes */
#define CONFIGURED_DRIVE			0x01 // configured drive - has meta data stamped on it so that it
 										 // can participate in array migrations/OCE
#define SMART_ERROR_OCCURRED		0x02 // drive has encountered SMART error
#define DRIVE_WRITE_CACHE_ENABLED	0x04 //drive write cache is enabled
#define DEVICE_TRANSPORT_TYPE_SATA	0x08 // Bit 4 set to one if SATA drive, set to zero if SAS drive

typedef struct _SCSIDeviceDef			/* structure size is 16 (0x12) bytes */
{
/* 0x0000 */	USHORT        SD_Device;		/* SCSI Id */
/* 0x0002 */	USHORT		  SD_LUN;
/* 0x0004 */	ULONG         SD_Capacity;		/* Useable capacity */
/* 0x0008 */	USHORT		  SD_SCSIDevType;
/* 0x000A */	UCHAR		  SD_Capability;	/* Indicates whether SMART support is available on this drive*/
/* 0x000B */	UCHAR         SD_State;			/* Ok, failed, etc. */
/* 0x000C */	ULONG		  SD_Speed;			/* Max speed Target supports */
/* 0x0010 */	ULONG		  SD_CurrentSpeed;	/* Negotiated Speed */
/* 0x0014 */	UCHAR		  SD_SOffset;
/* 0x0015 */	UCHAR		  SD_BWidth;
/* 0x0016 */	USHORT		  Padding;			/* padding to dword alignment */
} SCSIDeviceDef, DPTR pSCSIDeviceDef;

/* This is not used, we use PAGE_DEVICE_DEF instead to get the		*/
/* physical device information on each adapter.					*/
/*
#define PAGE_ADAPTER_PHYS_DEF 0x02
*/
#define SAS_ADDRESS_SIZE 8

/* Define for SD_Speed and SD_CurrentSpeed */
#define SAS_SPEED_15GBPS	0
#define SAS_SPEED_30GBPS	1


/* SAS support */
typedef struct _SASDeviceDef /* structure size is 32 (0x20) bytes */
{
/* 0x0000 */    UCHAR           SD_MappedPathId ;
/* 0x0001 */    UCHAR           SD_MappedTargetId ;
/* 0x0002 */    UCHAR           SD_MappedLun ;
/* 0x0003 */    UCHAR           SD_ScsiDeviceType ;   /* Tape, Disk etc*/
/* 0x0004 */    UCHAR           SD_Capability;        /* Indicates whether SMART support is available on this drive*/
/* 0x0005 */    UCHAR           SD_State;             /* Ok, failed, etc. */
/* 0x0006 */    UCHAR           SD_Speed;             /* Coded Max speed Target supports */
/* 0x0007 */    UCHAR           SD_CurrentSpeed;       /* Coded Current negotiated phy link speed */
/* 0x0008 */    UCHAR           SD_ExtendedAttributes; /* padding to dword alignment */
                                                       /* Bit 0 - CONFIGURED_DRIVE */
                                                       /* Bit 1 - SMART ERROR      */
                                                       /* Bit 2 - Drive Write Cache enabled */
/* 0x0009 */    UCHAR           SD_PortNumber ;	   	   /* The logical port number this SAS device is attached to */
/* 0x000a */    UCHAR           SD_Future [6] ;

/* 0x0010 */    ULONG           SD_Capacity ;          /* Useable capacity */
/* 0x0014 */    ULONG           SD_CapacityHigh ;      /* For future to help > 2TB capacity */
/* 0x0018 */    UCHAR           SD_SasWorldWideID [SAS_ADDRESS_SIZE] ;
/* 0x0020 */

} SASDeviceDef, DPTR pSASDeviceDef;

/* Definition of AP_Type
 * If this contains 0 (due to classical padding) treat it as PHYS_SCSI2 
 * and use SCSIDeviceDef structure. The AP_Type will contain one of these enumerations
 * AP_Type is defined in sysdef.h
 */

#define PAGE_DEVICE_DEF	0x01

#define PAGE_DEVICE_SUBPAGE0 0
#define PAGE_DEVICE_SUBPAGE1 1
#define PAGE_DEVICE_SUBPAGE2 2
#define PAGE_DEVICE_SUBPAGE3 3

#define PHYSDEV_SUBPAGE0_START_INDEX 0
#define PHYSDEV_SUBPAGE1_START_INDEX 32
#define PHYSDEV_SUBPAGE2_START_INDEX 64
#define PHYSDEV_SUBPAGE3_START_INDEX 96

#ifndef CDA_FOR_SATA
#define MAX_DEVICE_PER_PHYSDEV_PAGE	32
#else
// This can not be bigger than CDA_MAX_DEVICES_PER_ADAPTER, so set it to that value
#define MAX_DEVICE_PER_PHYSDEV_PAGE	CDA_MAX_DEVICES_PER_ADAPTER
#endif

typedef struct _AdapterPhysDevicePage 		/* structure size is 32 bytes */
{
 /* 0x0000 */	USHORT				cAP_PhysDevices;
 /* 0x0002 */   UCHAR				AP_Type; /* Type of  AdapterPhysDevicePage */
 /* 0x0003 */   UCHAR				AP_Size; /* Size of  AP_Device */
 /* 0x0004 */
			  union
#ifdef SCO_DDI8
               {
                  SCSIDeviceDef  _AP_Device[1];
                  SASDeviceDef   _AP_DeviceSAS[1];
               } _AdapterPhysDevicePage_union;
#define AP_Device 	_AdapterPhysDevicePage_union._AP_Device
#define AP_DeviceSAS 	_AdapterPhysDevicePage_union._AP_DeviceSAS
#else 	/* SCO_DDI8 */
               {
                  SCSIDeviceDef  AP_Device[1];
                  SASDeviceDef   AP_DeviceSAS[1];
               };
#endif 	/* SCO_DDI8 */
 
} AdapterPhysDevicePage, DPTR pAdapterPhysDevicePage;



/***********************************************************************
	MODE PAGES FOR A STORAGE DEVICE
	PAGE_DEVICE_DEF defines the phys devices that make up a logical device.
	PAGE_FUNC_DEF defines the functions currently being done on a device.
***********************************************************************/



typedef struct _LogicalMemberPage 			/* structure size is 88 (0x58) bytes */
{								   	
 /* 0x0000 */	USHORT		LM_State;	  			/* Ok, failed, etc. 		 */
 /* 0x0002 */	USHORT		LM_PhysType;			/* Define type interface, etc*/
 /* 0x0004 */	union _LM_Device
				{
					PhysicalDevicePage		LM_PhysDef;
					LDTree					LM_Branch;
				 /*	LogicalArrayPage		LM_LogDef; */
				} LM_Device;
} LogicalMemberPage, DPTR pLogicalMemberPage;

/* Define the page that describes the arrays from the device that controls them. */

typedef struct _LogicalArrayPage 			/* structure size is 196 (0xC4) bytes */
{
/* 0x0000 */	ULONG		LA_Size;					/* Size in bytes of WHOLE struc	*/
/* 0x0004 */	UCHAR		LA_DType;					/* SCSI Device type 				*/
/* 0x0005 */	UCHAR		LA_Status;					/* Ok, recovery, failed			*/
/* 0x0006 */	USHORT		LA_LocallyDefined;			/* nonzero if all members are here */
/* 0x0008 */	ULONG		LA_ArrayId;					/* Array unique identifier		*/
/* 0x000C */	ULONG		LA_Timestamp;				/* When config last changed	*/
/* 0x0010 */	USHORT		LA_PhysDriveIndex;			/* 0-based, what dev wants as index */
/* 0x0012 */	USHORT		LA_NumMembers;				/* Number of members in unit	*/
/* 0x0014 */	USHORT		LA_NumHeads;				/* Reported number of heads on device */
/* 0x0016 */	USHORT		LA_NumSPT;					/* Reported number of sectors/track */
/* 0x0018 */	USHORT		LA_NumCyl;					/* Reported cylinder count */
/* 0x001A */	USHORT		LA_StripingFactor;			/* Total addressibility 		*/
/* 0x001C */	USHORT		LA_BlocksPerCacheBlock;		/* Size of sector vs cache block	*/
/* 0x001E */	BOOL		LA_WriteBack; 				/* True for writeback, false for write thru */
/* 0x0020 */	ULONG		LA_TotalBlocks;
/* 0x0024 */	LogicalRedundancyPage	LA_RPage;
/* 0x006C */	LogicalMemberPage		LA_MemberData[1];	/* per-member info.				*/	
} LogicalArrayPage, DPTR pLogicalArrayPage;


#define PAGE_FUNC_DEF		0x00
#define FUNC_NONE			0x00
#define FUNC_VERIFY		0x01
#define FUNC_REBUILDING	0x02

/* Show the relative ranges of priorities wrt the function. The priority of*/
/* the function is how resources are to be allocated for RAID management	*/
/* functions.																*/

#define PRIORITY_RATIO		5
#define PRIORITY_HIGH		8
#define PRIORITY_MEDIUM		16	
#define PRIORITY_LOW		64
#define PRIORITY_DEFAULT	PRIORITY_MEDIUM

typedef struct _DeviceFuncDef		/* structure size is 84 (0x54) bytes */
{
/* 0x0000 */	UCHAR		DF_Function;
/* 0x0001 */	UCHAR		DF_Priority;
/* 0x0002 */	USHORT		Padding;			/* padding to dword alignment */
/* 0x0004 */	ULONG		DF_EndPoint;
/* 0x0008 */	ULONG		DF_CrntPoint;
/* 0x000C */	USHORT		DF_ArrayState;
/* 0x000E */	USHORT		DF_ArraySubState;
/* 0x0010 */	USHORT		DF_ArrayAccessType;
/* 0x0012 */	USHORT		DF_Convertible; /* Non zero if convertible 0 to 5 */
/* 0x0014 */	USHORT		DF_PhysState[5];
/* 0x0016 */	USHORT		Padding2;   		/* padding to dword alignment */
} DeviceFuncDef, DPTR pDeviceFuncDef;

typedef struct _DeviceFuncPage 	/* structure size is 88 (0x58) bytes */
{
/* 0x0000 */	USHORT			cDF_Units;
/* 0x0002 */	USHORT			Padding;			/* padding to dword alignment */
/* 0x0004 */	DeviceFuncDef	DF_Def[1];
} DeviceFuncPage, DPTR pDeviceFuncPage;

/* If possible, meaning in Intel world, some boot time code will place this	*/
/* structure in memory with a header that is fairly impossible to have occur	*/
/* randomly. Runtime drivers scan looking for this information. If it is		*/
/* encountered, then a driver configuration structure is created to allow for	*/
/* locating PCI devices without having to use configuration cycles. This prevents*/
/* drivers from possibly causing faults due to changes in the PCI defintiion of*/
/* how configuration cycles are to be performed. We conform to storing the		*/
/* information in the form of mode pages.										*/
/* 																			*/
/* Although the data structure supports the definition of devices that could be*/
/* included, we expect to have only controller structures here. All the		*/
/* storage devices are hidden behind the controller. A truely smart BIOS could	*/
/* create a list that has all the devices stored behind it.					*/

#define	MAP_SIG_1	0xCAFEFACEL
#define	MAP_SIG_2	0xDEADBEEFL
#define	MAP_SIG_3	0xCAFEBEEFL
#define	MAP_SIG_4	0XDEADFACEL

typedef struct _PCICfgHeader 			/* structure size is 84 (0x54) bytes */
{
/* 0x0000 */	ULONG					MapSig1;
/* 0x0004 */	ULONG					MapSig2;
/* 0x0008 */	ULONG					MapSig3;
/* 0x000C */	ULONG					MapSig4;
/* 0x0010 */	ControllerConfigPage	CPList;
} PCICfgHeader, DPTR pPCICfgHeader;

typedef struct _ConfigList DPTR pConfigList;
typedef struct _ConfigList 			/* structure size is 8 (0x08) bytes */
{
/* 0x0000 */	pPCICfgHeader	pCrntImage;
/* 0x0004 */	pConfigList		pNextImage;
} ConfigList;


/* Following datastructures are the SCSI Standard MODE SELECT/MODE	*/
/* SENSE data format for DISKs (DIRECT ACCESS DEVICE) taken from	*/
/* SCSI specification. Therefore, DO NOT modify any fields!!!		*/

typedef struct _ModeParmHeader6		/* structure size is 4 (0x04) bytes */
{
/* 0x0000 */	UCHAR	MP_DataLen;
/* 0x0001 */	UCHAR	MP_Medium;
/* 0x0002 */	UCHAR	MP_DeviceParm;
/* 0x0003 */	UCHAR	MP_BlkDescLen;
} ModeParmHeader6, DPTR pModeParmHeader6;

typedef struct _ModePage1C				/* structure size is 12 (0x0C) bytes */
{
/* 0x0000 */	UCHAR	MP_PageCode;
/* 0x0001 */	UCHAR	MP_PageLen;
/* 0x0002 */	struct 
				{
					UCHAR	LogErr:1;
					UCHAR	Reserved1:1;
					UCHAR	Test:1;
					UCHAR	DExcpt:1;
					UCHAR	Reserved2:3;
					UCHAR	Perf:1;
				} MP_Byte2;
/* 0x0003 */	struct
				{
					UCHAR 	MRIE:4;
					UCHAR 	Reserved:4;
				} MP_Byte3;				
/* 0x0004 */	UCHAR	MP_IntervalTimer[4];
/* 0x0008 */	UCHAR 	MP_ReportCount[4];
} ModePage1C, DPTR pModePage1C;

typedef struct _ModePage08				/* structure size is 12 (0x0C) bytes */
{
/* 0x0000 */	UCHAR	MP_PageCode;
/* 0x0001 */	UCHAR	MP_PageLen;
/* 0x0002 */	struct
				{
					UCHAR	RCD:1;
					UCHAR	MF:1;
					UCHAR 	WCE:1;
					UCHAR	Reserved:5;
				} MP_Byte2;
/* 0x0003 */	struct
				{
					UCHAR	WritePriority:4;
					UCHAR	ReadPriority:4;
				} MP_Byte3;
/* 0x0004 */	USHORT	DisablePreFetchLength;
/* 0x0006 */	USHORT	MinPreFetch;
/* 0x0008 */	USHORT	MaxPreFetch;
/* 0x000A */	USHORT	MaxPreFetchCeiling;
} ModePage08, DPTR pModePage08;

/*
 * Initiator Response Timeout in Mode Page 19
 */
#define MODE_PAGE19_IRTO    (10*1000)     /* 10 seconds */

typedef struct _ModePage19				/* structure size is 8 bytes */
{
/* 0x0000 */	UCHAR	MP_PageCode;
/* 0x0001 */	UCHAR	MP_PageLen;
/* 0x0002 */	struct
				{
					UCHAR	Protocol_ID:4;
					UCHAR	RLM:1;          /* Ready Led Meaning */
					UCHAR	Reserved:3;
				} MP_Byte2;
/* 0x0003 */	UCHAR   Reserved;
/* 0x0004 */	USHORT	I_TNexusLossTime;   /* in big endian order: MSB - LSB  */
/* 0x0006 */	USHORT	InitiatorResponseTimeout;
} ModePage19, DPTR pModePage19;

/* SCSI Mode Sense - Error Recovery Page(0x01) */
typedef struct _ModePage01				/* structure size is 12 (0x0C) bytes */
{
/* 0x0000 */	UCHAR	MP_PageCode;
/* 0x0001 */	UCHAR	MP_PageLen;
/* 0x0002 */	union {
			UCHAR ErrRecParam2_byte;
			struct {
				UCHAR	DCR:1;
				UCHAR	DTE:1;
				UCHAR 	PER:1;
				UCHAR 	EER:1;
				UCHAR 	RC:1;
				UCHAR 	TB:1;
				UCHAR 	ARRE:1;
				UCHAR 	AWRE:1;
			}ErrRecParam2_bits;
		}MP_Byte2;
/* 0x0003 */	UCHAR	RdRetryCnt;
/* 0x0004 */	UCHAR	CorrSpan;
/* 0x0005 */	UCHAR	HeadOffCnt;
/* 0x0006 */	UCHAR	DataStrbOffCnt;
/* 0x0007 */	UCHAR	Reserved1;
/* 0x0008 */	UCHAR	WrRetryCnt;
/* 0x0009 */	UCHAR	Reserved2;
/* 0x000A */	USHORT	RecoveryTimeLim;
} ModePage01, DPTR pModePage01;
/* Mode Parameter List Data Format for MODE SENSE/MODE SELECT consists of:*/
/* 	Mode Parameter Header											   */
/* 	Block Descriptor(s)												   */
/* 	Page(s)															   */
/* For DIRECT ACCESS DEVICE, it does not make sense to use any Block	   */
/* Descriptor; therefore, only Mode Page Format follows after Header.	   */
/* Below is the Mode Page Code 1C, which is used to turn on			   */
/* SMART option for disk drives.										   */
typedef struct _ModeData1C				/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */	ModeParmHeader6		MD_MPHeader6;
/* 0x0004 */	ModePage1C		MD_Page1C;
} ModeData1C, DPTR pModeData1C;

/* Below is the Mode Page Code 08h for Caching Page */
typedef struct _ModeData08				/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */	ModeParmHeader6		MD_MPHeader6;
/* 0x0004 */	ModePage08			MD_Page08;
} ModeData08, DPTR pModeData08;

/* Mode Page Code 19h for I_T Nexus Loss Time and Initiator Response Timeout Page */
typedef struct _ModeData19				/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */	ModeParmHeader6		MD_MPHeader6;
/* 0x0004 */	ModePage19			MD_Page19;
} ModeData19, DPTR pModeData19;

/* Mode Page Data for Page Code 01h for Error Recovery Page */
typedef struct _ModeData01				/* structure size is 16 (0x10) bytes */
{
/* 0x0000 */	ModeParmHeader6		MD_MPHeader6;
/* 0x0004 */	ModePage01		MD_Page01;
} ModeData01, DPTR pModeData01;
/* Included these definitions again so that IOMgr would compile.
 * But, these IOCtls won't be supported in the driver. In future
 * we can remove it when we remove references to these in IOMgr 
 * also.
 */
/*******************/ 
#define PAGE_CACHE_DEF			0x01

typedef struct _CacheDefinitionPage	/* structure size is 8 (0x08) bytes */
{
/* 0x0000 */	ULONG		szCD_CacheMemory;	/* KB of cache memory */
/* 0x0004 */	ULONG		szCD_CacheBlock;	/* KB per cache block */
} CacheDefinitionPage, DPTR pCacheDefinitionPage;

#define PAGE_STORAGE_DEF		0x02

#define CAP_PROFILING				0x1000L
#define CAP_CACHEING				0x2000L
#define CAP_RESERVED1_FOR_IOM		0x8000L

/*******************/ 

#endif
