/**********************************************************************
Filename:osmif.c

File Description
    OSM-provided routines required by CHIM.

    This is a core file actually but a link between the low level
    CHIM calls to call OS specific things such as PIC config space
    access and general port access routines.

***********************************************************************

History :
5/11/2005 KenC
1. OSMMapIOHandle: Updated baseAddress with required offset in the case of 
   HIM_IOSPACE. Previously, we only did this for HIM_MEMORYSPACE. Bug found
   by Judy Brock.
2. OSMWatchdog: Updated to leverage CDATimer by setting HIMWatchdogInterval to
   the total number of cda timer ticks instead of the discrete time asked by CHIM.

08/18/2004 Madhuresh
OSMMapIOHandle is called from CdaRestartAdapter via HIMVerify during S1, S3, S4
cycles. This tries to allocate a new handle. Protecting from being called more than
once.

08/10/2004 14:16  TRUIZ  Tony Ruiz
OSMMapIOHandle was using a ULONG for the base address being past to the CHIM.
This does not work on MemMap 64bit.

07/23/2004 14:16  NAR20038  Manjunath
Added OSMIobEvent to list of OSM provided routines.

11/22/2003 Madhuresh
OSMDelay is replaced by XMicrosecondDelay.

10/31/2003 Madhuresh
For SAS, moved OSMEvent to physif.c. Used HIM_INCLUDE_SAS to remove such
functions not needed in SAS or to change OSMAdjustBusAddress. PCI Config
space reads, writes now send device extension instead of the pAD. Port I/O
routines read write conditionally to both I/O and memory mapped spaces now.

01/14/2002 16:30 Madhuresh
Modified all Port IO routines to optimize the undesirable checks on flags.
Cleaned up MapIOHandle so it supports more than one IO ranges as required
by U320 CHIM.

12/07/2000 12:53 AM   NAR20038  Manjunath
Changes from CDA to TajMahal code base. For previous history please 
refer the file in Build 1.

**********************************************************************/
#if defined(__linux__) || defined(SCO_DDI8)
#include "hr.h"
#endif /* __linux__ */

#ifdef NETWARE
#include "nw.h"
#endif

#include "sysdef.h"

/*
    If the CPU and the IO Processor do NOT have same ENDIAN format,
    we need to swap the bytes for 16 and 32 bit I/Os to ports or memory.
    The following macro figures out that need.
*/
#define NO_ENDIAN_SWAP  ((OSM_CPU_LITTLE_ENDIAN && HIM_IO_LITTLE_ENDIAN) || (!OSM_CPU_LITTLE_ENDIAN && HIM_IO_BIG_ENDIAN))


/*
if ((OSM_CPU_LITTLE_ENDIAN && (handle->attributes & HIM_IO_LITTLE_ENDIAN)) ||
        ((!OSM_CPU_LITTLE_ENDIAN) && (handle->attributes & HIM_IO_BIG_ENDIAN)) ||
        ((handle->attributes &  (HIM_IO_LITTLE_ENDIAN | HIM_IO_BIG_ENDIAN)) == 0))
    {
        swapping = HIM_FALSE;
    } 
    else
    {
        swapping = HIM_TRUE;
    }
*/
/**********************************************************************

Function Name: OSMMapIOHandle

Function Description
    This routine is to provide handles to I/O ranges needed to access
    adapter registers and on-board memory.

CAUTION:
    This routine should be called as many times as CHIM has reported
    the number of MaxIOHandles. Calling it more times, will return an
    error HIM_FAILURE.

    The handles are basically a four field data structure containing the
    base address, and some flags. The base address is what is used for
    performing the PORT I/Os or memory mapped IOs.

Input Parameters
    rangeIndex  The range number requested.  For PCI, this is referring
                to the six supported I/O ranges in PCI configuration space.
    offset      Offset within range to be mapped.
    length      Number of bytes to be mapped.
    pacing      Number of microseconds between consecutive accesses to any
                offset within the mapped range
    attributes  Describe the I/O range to be mapped.  Refer to CHIM spec.
    handle      Used in future accesses to the specified I/O range.

Output Parameters   
    0           Handle is successfully returned.
    1           Unable to map this I/O range.

**********************************************************************/
HIM_UINT8
OSMMapIOHandle( void HIM_PTR pOSMContex,
                HIM_UINT8       rangeIndex,
                HIM_UINT32      offset,
                HIM_UINT32      length, 
                HIM_UINT32      pacing, 
                HIM_UINT16      attributes,
                HIM_IO_HANDLE   HIM_PTR handle)
{
    pAdapterDevice  pAD = pOSMContex;
    HIM_IO_HANDLE   newHandle;
    DATA_N_PTR      baseAddr; 
    USHORT          k;

    /*
        OSMMapIOHandle is called from CdaRestartAdapter via HIMVerify
        during S1, S3, S4 cycles. This tries to allocate a new handle.
        Protecting from being called more than once.
    */
    if( pAD->pAdapterDevExt->InitializationTime == FALSE )
        return HIM_SUCCESS ;

    /*
        Find an empty handle slot. Total of max io handles count of
        handle structure space has been allocated earlier.
    */
    newHandle = pAD->AD_IOHandles;

    for (k=0 ; k < pAD->AD_MaxIOHandles ; k++ )
    {
        if (newHandle->handleUsed == HIM_FALSE)
        {
            newHandle->handleUsed = HIM_TRUE;
            break;
        }
        newHandle++; /* moves to next structure */
    }

    if (k >= pAD->AD_MaxIOHandles)
    {
        /*
            CHIM gave a lower number of MaxIOHandles earlier,
            than the number of rangeIndex he has refered this far.
        */
        return (HIM_FAILURE);
    }
    /*
        Find the base address and set flag if it is memory mapped.
    */
    baseAddr = (DATA_N_PTR) pAD->AD_BAR[ rangeIndex ].BaseAddr; 

    if (pAD->AD_BAR[rangeIndex].MemoryMapped == HIM_TRUE) 
    {
        newHandle->memoryMapped = HIM_MEMORYSPACE;
        newHandle->baseAddress  = (DATA_N_PTR) baseAddr ;
        /*  We need to use the offset according to CHIM spec */
        newHandle->baseAddress += offset;
     }
    else 
    {
        newHandle->memoryMapped = HIM_IOSPACE;
        newHandle->baseAddress  = (DATA_N_PTR) baseAddr ;
        newHandle->baseAddress += offset;
    }   

    newHandle->attributes = attributes;
    HIM_PTR handle = newHandle;

    return HIM_SUCCESS;
}
/**********************************************************************

Function Name: OSMReleaseIOHandle

Function Description
    This routine releases memory and other resources that are associated
    with an obsolete HIM_IO_HANDLE.

Input Parameters
    pOSMContext     The OSM-generated identifier for an adapter.

Output Parameters   
    0               Success
    Non-0           Added in the future.

**********************************************************************/
HIM_UINT8
OSMReleaseIOHandle(
    VOID HIM_PTR    pOSMContext,
    HIM_IO_HANDLE   handle)
{
    /* Mark handle as false.  Note that all handles are
     * currently acquired at once, so releasing a handle
     * isn't usually helpful, unless HIM acquires AND
     * releases handles within HIMVerifyAdapter.  This
     * sequence is mostly for the possibility of handles
     * being more dynamic than currently defined.
     */
    pOSMContext = pOSMContext;
    handle->handleUsed = HIM_FALSE;

    return HIM_SUCCESS;
}
/**********************************************************************

Function Name: OSMAdjustBusAddress

Function Description
    This routine adds or substracts a value to a HIM_BUS_ADDRESS.
    NOTE: We have to add to pointers for the HIM, because HIM doesn't 
    know the exact type.  This is an issue for 64-bit addresses.
    For 32 bits (which this assumes), it's rather trivial.

Input Parameters
    busAddress      a HIM bus address
    value           a signed value to be added to bus address.

Output Parameters   
    None

**********************************************************************/
void
OSMAdjustBusAddress(
    HIM_BUS_ADDRESS    HIM_PTR busAddress,
    HIM_SINT32         value)
{
    /*
        Now add or subtract 64 bit number.
    */
    busAddress->u32entity0 += value ;
    if( busAddress->u32entity1 != 0)
        CDA_ASSERT(FALSE);
}

/**********************************************************************

Function Name: OSMReadUExact8
                OSMReadUExact16
                OSMReadUExact32
                
Function Description
    These routines read from an I/O range which the OSM has previously
    mapped for the HIM

Input Parameters
    Handle      An OSM-specified handle, acquired through OSMMapIOHandle.
    Offset      add to the base address.

Output Parameters   
    Data value at requested based and offset.

**********************************************************************/
HIM_UEXACT8 OSMReadUExact8(
    HIM_IO_HANDLE   Handle, 
    HIM_UINT32      Offset)
{
#ifdef LINUX
#ifdef CDA_FOR_SATA
    HIM_UEXACT8     val;

    if( Handle->memoryMapped == HIM_IOSPACE )
        val = INBYTE(Handle->baseAddress + Offset);
    else
        val = readb(Handle->baseAddress + Offset);
            
    //val = HIM_PTR(HIM_UEXACT8 HIM_PTR)(Handle->baseAddress + Offset);

    mb();

    return (val);
#else /* CDA_FOR_SATA */
    if( Handle->memoryMapped == HIM_IOSPACE )
        return( INBYTE(Handle->baseAddress + Offset) ) ;
    else
        return( HIM_PTR(HIM_UEXACT8 HIM_PTR)(Handle->baseAddress + Offset) ) ;
#endif /* CDA_FOR_SATA */
#else
    if( Handle->memoryMapped == HIM_IOSPACE )
        return( INBYTE(Handle->baseAddress + Offset) ) ;
    else
        return( HIM_PTR(HIM_UEXACT8 HIM_PTR)(Handle->baseAddress + Offset) ) ;
 #endif /* LINUX */
}
/**********************************************************************/
HIM_UEXACT16 OSMReadUExact16(
    HIM_IO_HANDLE   Handle, 
    HIM_UINT32      Offset)
{
#if NO_ENDIAN_SWAP
#ifdef LINUX
#ifdef CDA_FOR_SATA
    HIM_UEXACT16    val;
 
    if( Handle->memoryMapped == HIM_IOSPACE )
        val = INWORD(Handle->baseAddress + Offset);
    else
        val = readw(Handle->baseAddress + Offset);

        //val = (HIM_PTR(HIM_UEXACT16 HIM_PTR)(Handle->baseAddress + Offset));

    mb();
    return (val);
#else /* CDA_FOR_SATA */
    if( Handle->memoryMapped == HIM_IOSPACE )
        return ( INWORD(Handle->baseAddress + Offset) ) ;
    else
        return ( (HIM_PTR(HIM_UEXACT16 HIM_PTR)(Handle->baseAddress + Offset)) ) ;
#endif /* CDA_FOR_SATA */
#else
 /* LINUX */
    if( Handle->memoryMapped == HIM_IOSPACE )
        return ( INWORD(Handle->baseAddress + Offset) ) ;
    else
        return ( (HIM_PTR(HIM_UEXACT16 HIM_PTR)(Handle->baseAddress + Offset)) ) ;


#endif /* LINUX */
#else
    {
        HIM_UEXACT16            rc;

        if( Handle->memoryMapped == HIM_IOSPACE )
            rc = INWORD(Handle->baseAddress + Offset) ;
        else
            rc = (HIM_PTR(HIM_UEXACT16 HIM_PTR)(Handle->baseAddress + Offset) ) ;

        return(((rc & 0xff) << 8) + ((rc & 0xff00) >> 8)) ;
    }
#endif

}
/**********************************************************************/
HIM_UEXACT32    OSMReadUExact32(
    HIM_IO_HANDLE   Handle, 
    HIM_UINT32      Offset)
{

#if NO_ENDIAN_SWAP
#ifdef LINUX
#ifdef CDA_FOR_SATA
    HIM_UEXACT32    val;
    if( Handle->memoryMapped == HIM_IOSPACE )
        val = INDWORD(Handle->baseAddress + Offset);
    else
        val = readl(Handle->baseAddress + Offset);
    //val = (HIM_PTR(HIM_UEXACT32 HIM_PTR)(Handle->baseAddress + Offset));
    mb();
    return (val);
#else /* CDA_FOR_SATA */
    if( Handle->memoryMapped == HIM_IOSPACE )
        return( INDWORD(Handle->baseAddress + Offset) ) ;
    else
        return ( (HIM_PTR(HIM_UEXACT32 HIM_PTR)(Handle->baseAddress + Offset)) ) ;
#endif /* CDA_FOR_SATA */
#else  /* LINUX */
    if( Handle->memoryMapped == HIM_IOSPACE )
        return( INDWORD(Handle->baseAddress + Offset) ) ;
    else
        return ( (HIM_PTR(HIM_UEXACT32 HIM_PTR)(Handle->baseAddress + Offset)) ) ;

#endif /* LINUX */
#else  /* NO_ENDIAN_SWAP */
    {
        HIM_UEXACT32    rc ;

        if( Handle->memoryMapped == HIM_IOSPACE )
            rc = INWORD(Handle->baseAddress + Offset) ;
        else
            rc = (HIM_PTR(HIM_UEXACT32 HIM_PTR)(Handle->baseAddress + Offset) ) ;

        return(((rc & 0xff000000) >> 24) + ((rc & 0x00ff0000) >>  8) + 
               ((rc & 0x0000ff00) <<  8) + ((rc & 0x000000ff) << 24));
    }
#endif  /* NO_ENDIAN_SWAP */
}
/**********************************************************************
Function Name: OSMReadStringUExact8
                OSMReadStringUExact16
                OSMReadStringUExact32
                
Function Description
    Similar to OSMReadUEXACT8, etc., except that these routines will 
    return multiple data of the requested size into a HIM-specified
    buffer.

Input Parameters
    Handle       An OSM-specified handle, acquired through OSMMapIOHandle.
    Offset       add to the baseaddress.
    destBuffer   Data buffer specified by HIM.
    count        Number of HIM_UEXACR8s, 16s, 32s, to be read.
    stride       Number of bytes between data units on the handle-controlled source

Output Parameters
    None

**********************************************************************/
VOID OSMReadStringUExact8(
    HIM_IO_HANDLE       Handle,
    HIM_UINT32          Offset,
    HIM_UEXACT8 HIM_PTR destBuffer, 
    HIM_UINT32          count, 
    HIM_UINT8           stride)
{
    HIM_UINT32              strideIndex; /* applies to i/o range only */
    HIM_UINT32              index;

    /* We will need to support HIM_IOSPACE and HIM_MEMORYSPACE for later CHIM. */
    
    for (strideIndex = 0, index = 0 ; index < count ; strideIndex += stride , index++ )
    {
        if( Handle->memoryMapped == HIM_IOSPACE )
            HIM_PTR destBuffer = INBYTE(Handle->baseAddress + Offset + strideIndex) ;
        else
            HIM_PTR destBuffer = HIM_PTR(HIM_UEXACT8 HIM_PTR) (Handle->baseAddress + Offset + strideIndex) ;

        destBuffer++;
    }
}
/**********************************************************************/
VOID    OSMReadStringUExact16(
    HIM_IO_HANDLE         Handle, 
    HIM_UINT32            Offset,
    HIM_UEXACT16 HIM_PTR  destBuffer, 
    HIM_UINT32            count, 
    HIM_UINT8             stride)
{
    HIM_UINT32    strideIndex; /* applies to i/o range only */
    HIM_UINT32    index;
    HIM_UEXACT16  value;


    for (strideIndex = index = 0 ; index < count ; strideIndex += stride , index++ )
    {
        if( Handle->memoryMapped == HIM_IOSPACE )
            value = INWORD(Handle->baseAddress + Offset + strideIndex) ;
        else
            value = HIM_PTR(HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset + strideIndex) ;

#if NO_ENDIAN_SWAP
            HIM_PTR destBuffer = value;
#else
            HIM_PTR destBuffer = ((value & 0xff) << 8) + ((value & 0xff00) >> 8);
#endif
    
        destBuffer++;
    }
}
/**********************************************************************/
VOID OSMReadStringUExact32(
    HIM_IO_HANDLE    Handle, 
    HIM_UINT32       Offset,
    HIM_UEXACT32     HIM_PTR destBuffer, 
    HIM_UINT32       count, 
    HIM_UINT8        stride)
{
    HIM_UINT32    strideIndex;
    HIM_UINT32    index;
    HIM_UEXACT32  value;

    for (strideIndex = index = 0 ; index < count ; strideIndex += stride , index++ )
    {
        if( Handle->memoryMapped == HIM_IOSPACE )
            value = INDWORD(Handle->baseAddress + Offset + strideIndex) ;
        else
            value = HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset + strideIndex) ;

#if NO_ENDIAN_SWAP

        HIM_PTR destBuffer = value ;
#else
        HIM_PTR destBuffer = ((value & 0xff000000) >> 24) + ((value & 0x00ff0000) >>  8) + 
                    ((value & 0x0000ff00) <<  8) + ((value & 0x000000ff) << 24) ;
#endif
        destBuffer++;
    }
}
/**********************************************************************

Function Name: OSMWriteUExact8
                OSMWriteUExact16
                OSMWriteUExact32
                
Function Description
    These routines write to an I/O range which the OSM has previously
    mapped for the HIM.

Input Parameters
    Handle    An OSM-specified handle, acquired through OSMMapIOHandle.
    Offset    add to baseaddress.
    value     Data to be written.

Output Parameters
    None

**********************************************************************/
VOID    OSMWriteUExact8(
    HIM_IO_HANDLE    Handle, 
    HIM_UINT32       Offset, 
    HIM_UEXACT8      value)
{
    if( Handle->memoryMapped == HIM_IOSPACE )
        OUTBYTE(Handle->baseAddress + Offset, value) ;
    else
#ifdef LINUX
#ifdef CDA_FOR_SATA
        writeb(value, (Handle->baseAddress + Offset));
        //HIM_PTR(HIM_UEXACT8 HIM_PTR) (Handle->baseAddress + Offset) = value ;
    mb();
#else /* CDA_FOR_SATA */
        HIM_PTR(HIM_UEXACT8 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* CDA_FOR_SATA */
#else /* LINUX */
        HIM_PTR(HIM_UEXACT8 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* LINUX */
}
/**********************************************************************/
VOID    OSMWriteUExact16(
    HIM_IO_HANDLE   Handle,
    HIM_UINT32      Offset,
    HIM_UEXACT16    value)
{
#if NO_ENDIAN_SWAP
    if( Handle->memoryMapped == HIM_IOSPACE )
        OUTWORD(Handle->baseAddress + Offset, value) ;
    else
#ifdef LINUX
#ifdef CDA_FOR_SATA
        writew(value, (Handle->baseAddress + Offset));
        //HIM_PTR(HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#else /* CDA_FOR_SATA */
    HIM_PTR(HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* CDA_FOR_SATA */
#else /* LINUX */
        HIM_PTR(HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* LINUX */
#else
    if( Handle->memoryMapped == HIM_IOSPACE )
        OUTWORD(Handle->baseAddress + Offset, ((value & 0xff00) >> 8) + ((value & 0x00ff) << 8)) ;
    else
        HIM_PTR(HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset) = ((value & 0xff00) >> 8) + ((value & 0x00ff) << 8) ;
#endif
 /* NO_ENDIAN_SWAP */
#ifdef LINUX
#ifdef CDA_FOR_SATA
    mb();
#endif /* CDA_FOR_SATA */
#endif /* LINUX */
}
/**********************************************************************/
VOID    OSMWriteUExact32(
    HIM_IO_HANDLE   Handle, 
    HIM_UINT32      Offset, 
    HIM_UEXACT32    value)
{

#if NO_ENDIAN_SWAP
    if( Handle->memoryMapped == HIM_IOSPACE )
        OUTDWORD(Handle->baseAddress + Offset, value) ;
    else
#ifdef LINUX
#ifdef CDA_FOR_SATA
        writel(value, (Handle->baseAddress + Offset));
        //HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#else /* CDA_FOR_SATA */
        HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* CDA_FOR_SATA */
#else /* LINUX */
        HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset) = value ;
#endif /* LINUX */
#else
    HIM_UEXACT32  SwappedValue = ((value & 0xff000000) >> 24) + ((value & 0x00ff0000) >>  8) + 
                                 ((value & 0x0000ff00) <<  8) + ((value & 0x000000ff) << 24) ;

    if( Handle->memoryMapped == HIM_IOSPACE )
        OUTDWORD(Handle->baseAddress + Offset, SwappedValue ) ;
    else
        HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset) = SwappedValue ;
#endif
#ifdef LINUX
#ifdef CDA_FOR_SATA
    mb();
#endif /* CDA_FOR_SATA */
#endif /* LINUX */
}
/**********************************************************************

Function Name: OSMWriteStringUExact8
                OSMWriteStringUExact16
                OSMWriteStringUExact32
                
Function Description
    Similar to OSMWriteUEXACT8, etc., except that this routine will use
    a HIM-specified buffer as the source of multiple data.

Input Parameters
    Handle        An OSM-specified handle, acquired through OSMMapIOHandle.
    Offset        Offset, in bytes, within the range defined by handle.
    sourceBuffer  Source data buffer dpecified by HIM.
    count         Number of HIM_UEXACT8s, 16s, or 32s to be written.
    stride        Number of bytes between data units on the handle-controlled source.

NOTE:
'stride' refers to the destination, handle-controlled I/O range buffer.
The stride on the source buffer is always the size of of the string data element.
'stride' is always in terms of 8-bit bytes.  It is assumed to be 0,
or an integer multiple of the data type (this is more permissive than the spec's 
restrictions, which are 0 or sizeof data)

Output Parameters
    None
**********************************************************************/
VOID    OSMWriteStringUExact8(
    HIM_IO_HANDLE    Handle, 
    HIM_UINT32       Offset, 
    HIM_UEXACT8      HIM_PTR sourceBuffer, 
    HIM_UINT32       count, 
    HIM_UINT8        stride)
{
   HIM_UINT32  strideIndex; /* applies to i/o range only */
   HIM_UINT32  index;
    
   for (strideIndex = index = 0 ; index < count ; strideIndex += stride , index++ )
   {
        if( Handle->memoryMapped == HIM_IOSPACE )
            OUTBYTE(Handle->baseAddress + Offset + strideIndex, HIM_PTR sourceBuffer) ;
        else
            HIM_PTR(HIM_UEXACT8 HIM_PTR) (Handle->baseAddress + Offset + strideIndex) = HIM_PTR sourceBuffer ;

        sourceBuffer++ ;
   }
}
/**********************************************************************/
VOID    OSMWriteStringUExact16(
    HIM_IO_HANDLE    Handle,
    HIM_UINT32       Offset,
    HIM_UEXACT16     HIM_PTR sourceBuffer,
    HIM_UINT32       count,
    HIM_UINT8        stride)
{
    HIM_UINT32       strideIndex ; /* applies to i/o range only */
    HIM_UINT32       index ;
    HIM_UEXACT16     SwappedValue ;

   for (strideIndex = index = 0 ; index < count ; strideIndex += stride , index++ )
   {

#if NO_ENDIAN_SWAP

        SwappedValue = HIM_PTR sourceBuffer ;
#else
        SwappedValue = (((HIM_PTR sourceBuffer) & 0xff00) >>  8) + (((HIM_PTR sourceBuffer) & 0x00ff) <<  8);
#endif

        if( Handle->memoryMapped == HIM_IOSPACE )
            OUTWORD(Handle->baseAddress + Offset + strideIndex, SwappedValue);
        else
            HIM_PTR((HIM_UEXACT16 HIM_PTR) (Handle->baseAddress + Offset + strideIndex)) = SwappedValue ;

        sourceBuffer++;
   }
}

/**********************************************************************/
VOID    OSMWriteStringUExact32(
    HIM_IO_HANDLE    Handle,
    HIM_UINT32       Offset,
    HIM_UEXACT32     HIM_PTR sourceBuffer,
    HIM_UINT32       count,
    HIM_UINT8        stride)
{
    HIM_UINT32    strideIndex; /* applies to i/o range only */
    HIM_UINT32    index;
    HIM_UINT32    SwappedValue;
    
    for (strideIndex = index = 0 ; index < count ; strideIndex += stride , index++ )
    {
#if NO_ENDIAN_SWAP

        SwappedValue = HIM_PTR sourceBuffer;
#else
        SwappedValue = 
            (((HIM_PTR sourceBuffer) & 0xff000000) >> 24) + 
            (((HIM_PTR sourceBuffer) & 0x00ff0000) >>  8) + 
            (((HIM_PTR sourceBuffer) & 0x0000ff00) <<  8) + 
            (((HIM_PTR sourceBuffer) & 0x000000ff) << 24) ;

#endif

        if( Handle->memoryMapped == HIM_IOSPACE )
            OUTDWORD(Handle->baseAddress + Offset + strideIndex, SwappedValue) ;
        else
            HIM_PTR(HIM_UEXACT32 HIM_PTR) (Handle->baseAddress + Offset + strideIndex) = SwappedValue ;

        sourceBuffer++;
    }
}
/**********************************************************************

Function Name: OSMSynchronizeRange

Function Description
    This routine ensures that previous I/O range operations to the
    specified range have completed.  

Input Parameters
    handle      An OSM-specified handle, acquired through OSMMapIOHandle.
    ioOffset    Offset, in bytes, within the range defined by handle.
    length      Number of bytes in the range to be synchronized.    
    
Output Parameters   
    None

**********************************************************************/
VOID    OSMSynchronizeRange(
    HIM_IO_HANDLE   handle,
    HIM_UINT32      ioOffset, 
    HIM_UINT32      length)
{
}


/**********************************************************************

Function Name: OSMWatchDog

Function Description
    This routine registers or de-registers a routine to be serviced
    with a watch dog timer.

Input Parameters
   pOSMContext          The OSM-generated identifier for an adapter.
    watchdogProcedure   Pointer to routine to be serviced with watch dog
                        timer.
    microseconds        Interval count of microseconds between service calls.

Output Parameters   
    None

**********************************************************************/
VOID    OSMWatchdog(
    VOID HIM_PTR         pOSMContext,
    HIM_WATCHDOG_FUNC   watchdogProcedure,
    HIM_UINT32          microSeconds)
{
#ifdef CDA_FOR_SATA  
   HIM_UINT32			TotalTicks = 0;	
   pAdapterDevice       pAD  = (pAdapterDevice) pOSMContext;

   if (watchdogProcedure)
   { 
      /* save the HIM watchdog timer routine and service interval for this adapter */
      pAD->HIMWatchdogFunc = watchdogProcedure;
      // 
	  // Convert the OSMWatchdog timer to discrete timer ticks
	  // Currently Default is 2 ticks per sec. 
	  //
	  TotalTicks			   = (microSeconds/1000000) * pAD->pAdapterDevExt->TicksPerSec ;
	  if (TotalTicks == 0 ) {
		
			pAD->HIMWatchdogInterval = 1;
	  
	  } else {

			pAD->HIMWatchdogInterval = TotalTicks;
			//
			//  Increase the watchdog interval by 1 more tick if 
			//  there's a "remainder" in the interger division
			//  When we DO NOT want to return the watchdog timer
			//  to the CHIM before it is ready to process...
			//
			if ((microSeconds%1000000) * pAD->pAdapterDevExt->TicksPerSec)
				pAD->HIMWatchdogInterval++;
	  }

   }   
   else
   {
      /* null watchdog function parameter ==  request to cancel the periodic timer */
       pAD->HIMWatchdogFunc = (HIM_WATCHDOG_FUNC) 0;
	   pAD->HIMWatchdogInterval = 0;
   }

#endif /* CDA_FOR_SATA   */

} /* end OSMWatchdog */

/**********************************************************************

Function Name: OSMReadPCIConfigurationDword,Word,Byte

Function Description
    These routines read from PCI configuration space.

Input Parameters
    pOSMContext         Pointer to AdapterDevice.
    registerNumber      Offset within config space.
    
Output Parameters   
    Contents of the register read from PCI configuration space.

**********************************************************************/
HIM_UEXACT32    OSMReadPCIConfigurationDword(
    void HIM_PTR pOSMContext,
    HIM_UINT16       registerNumber)
{
    pAdapterDevice  pAD = pOSMContext;
    
    return ((HIM_UEXACT32)XOSMReadPCIConfigurationDword(pAD->pAdapterDevExt, (ULONG)registerNumber));
}
/**********************************************************************/
HIM_UEXACT16    OSMReadPCIConfigurationWord(
    void HIM_PTR pOSMContext,
    HIM_UINT16       registerNumber)
{
    pAdapterDevice  pAD = pOSMContext;
    return ((HIM_UEXACT16)XOSMReadPCIConfigurationWord(pAD->pAdapterDevExt, (ULONG)registerNumber));
}

/**********************************************************************/
HIM_UEXACT8 OSMReadPCIConfigurationByte(
    void HIM_PTR pOSMContext,
    HIM_UINT16       registerNumber)
{
    pAdapterDevice  pAD = pOSMContext;
    return ((HIM_UEXACT8)XOSMReadPCIConfigurationByte(pAD->pAdapterDevExt, (ULONG)registerNumber));
}
/**********************************************************************

Function Name: OSMWritePCIConfigurationDword,Word,Byte

Function Description
    These routines write to PCI configuration space.

Input Parameters
    pOSMContext         Pointer to AdapterDevice.
    registerNumber      Offset within config space.
    registerValue       Value to be written to PCI configuration
                        space register.
Output Parameters   
    None

**********************************************************************/
VOID    OSMWritePCIConfigurationDword(
    void HIM_PTR pOSMContext,
    HIM_UINT16        registerNumber,
    HIM_UEXACT32     registerValue)
{
    pAdapterDevice  pAD = pOSMContext;
    XOSMWritePCIConfigurationDword(pAD->pAdapterDevExt, (ULONG)registerNumber, (ULONG)registerValue);
}

/**********************************************************************/
VOID    OSMWritePCIConfigurationWord(
    void HIM_PTR pOSMContext,
    HIM_UINT16       registerNumber,
    HIM_UEXACT16    registerValue)
{
    pAdapterDevice  pAD = pOSMContext;
    XOSMWritePCIConfigurationWord(pAD->pAdapterDevExt, (ULONG)registerNumber, (USHORT)registerValue);
}
/**********************************************************************/
VOID    OSMWritePCIConfigurationByte(
    void HIM_PTR pOSMContext,
    HIM_UINT16 registerNumber,
    HIM_UEXACT8 registerValue)
{
    pAdapterDevice  pAD = pOSMContext;
    XOSMWritePCIConfigurationByte(pAD->pAdapterDevExt, (ULONG)registerNumber, (UCHAR)registerValue);
}
/**********************************************************************

Function Name: FillOutOSMPointers

Function Description
    Write out function pointer packet destinated for HIM.
    
Input Parameters
    osmRoutines     pointer to OSM routine structure that initialize 
                    all function pointers required by HIM.

    AD_MemOrIO      Is it memory mapped..we switch the functions if true
Output Parameters   
    None

**********************************************************************/
VOID FillOutOSMPointers(
    HIM_OSM_FUNC_PTRS HIM_PTR   osmRoutines,
    UCHAR                       AD_MemOrIO)
{
    XMemZero((UCHAR DPTR) osmRoutines,sizeof(HIM_OSM_FUNC_PTRS));
    osmRoutines->versionNumber =  0;
    osmRoutines->OSMMapIOHandle = OSMMapIOHandle;
    osmRoutines->OSMReleaseIOHandle = OSMReleaseIOHandle;
    /* OSMEvent is no longer supported by CHIM starting V330 */
    /*osmRoutines->OSMEvent = OSMEvent;  */
    osmRoutines->OSMAdapterEvent = OSMAdapterEvent;
    osmRoutines->OSMPortEvent = OSMPortEvent;
    osmRoutines->OSMIobEvent = OSMIobEvent;
    osmRoutines->OSMLuEvent = OSMLuEvent;
    osmRoutines->OSMReadUExact8 = OSMReadUExact8;
    osmRoutines->OSMReadUExact16 = OSMReadUExact16;
    osmRoutines->OSMReadUExact32 = OSMReadUExact32;
    osmRoutines->OSMReadStringUExact8 = OSMReadStringUExact8;
    osmRoutines->OSMReadStringUExact16 = OSMReadStringUExact16;
    osmRoutines->OSMReadStringUExact32 = OSMReadStringUExact32;
    osmRoutines->OSMWriteUExact8 = OSMWriteUExact8;
    osmRoutines->OSMWriteUExact16 = OSMWriteUExact16;
    osmRoutines->OSMWriteUExact32 = OSMWriteUExact32;
    osmRoutines->OSMWriteStringUExact8 = OSMWriteStringUExact8;
    osmRoutines->OSMWriteStringUExact16 = OSMWriteStringUExact16;
    osmRoutines->OSMWriteStringUExact32 = OSMWriteStringUExact32;
    osmRoutines->OSMSynchronizeRange = OSMSynchronizeRange;
    osmRoutines->OSMWatchdog = OSMWatchdog;
    osmRoutines->OSMReadPCIConfigurationDword = OSMReadPCIConfigurationDword;
    osmRoutines->OSMReadPCIConfigurationWord = OSMReadPCIConfigurationWord;
    osmRoutines->OSMReadPCIConfigurationByte = OSMReadPCIConfigurationByte;
    osmRoutines->OSMWritePCIConfigurationDword = OSMWritePCIConfigurationDword;
    osmRoutines->OSMWritePCIConfigurationWord = OSMWritePCIConfigurationWord;
    osmRoutines->OSMWritePCIConfigurationByte = OSMWritePCIConfigurationByte;
    osmRoutines->OSMDelay = XMicrosecondDelay;
    osmRoutines->OSMAdjustBusAddress = OSMAdjustBusAddress;
#ifdef CDA_FOR_SATA         
        osmRoutines->OSMsgCopyOut = OSMsgCopyOut;
        osmRoutines->OSMsgCopyIn = OSMsgCopyIn;
#endif
}
